// Global Header file of the CForth  project **********************************
//
// Author and (c)2020-24 Wolfgang Schemmert. The copyright claim is restricted
// to this file and the depending files "main.c", "main.h", "cforth.c",
// and "cforth.h"
// Other depending material is published, copyrighted and licensed
// by STMicroelectronics and Segger
//
// Contact: <www.midi-and-more.de>
// A detailled hardware construction and operation manual
// is provided at this website
//
// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************

#ifndef GLOBALDEFINES
#define GLOBALDEFINES

#define REVNUM  19			//describes firmware revision
#define HARDWARE_VERSION 1	//describes project specific hardware version

//#define CPU_CLOCK144
//ifndef: CPU Clock is 48 MHz (default)

//the device will appear as "STMicroelectronics Virtual Com Port"
#define DEFAULT_VIDH	0x04
#define DEFAULT_VIDL	0x83
#define DEFAULT_VID		((DEFAULT_VIDH << 8) | DEFAULT_VIDL)
#define DEFAULT_PIDH	0x57
#define DEFAULT_PIDL	0x40
#define DEFAULT_PID		((DEFAULT_PIDH << 8) | DEFAULT_PIDL)

//attributes of KernelOperators
//attribute entries are merged with some other functional flags !!
#define A_IME			0x10000 //ATTRIBUTE_IMMEDIATE, cannot be compiled
#define A_CPO			0x20000 //ATTRIBUTE_COMPILEONLY,refuse work interpreting
#define A_BAK			0x40000 //may be parsed in background at executing state
#define A_CI			(A_CPO | A_IME) //COMPILEONLY IMMEDIATE
								//performs complex operations during compilation
#define A_IB			(A_IME | A_BAK)
#define A_RUN			0x80000 //ATTRIBUTE_RUNTIME,
					//compiled by other operators for special runtime behaviour
					//no direct user access, essential for viewing SEE and STEP
//only attribute of UserToken
#define A_VARCON		0x0C000000

#define ERR_STRINGTOKEN	1
#define ERR_SYNTAX		2
#define ERR_STACK		4
#define ERR_COMPONLY	8
#define ERR_SYSTEM		0x10
#define ERR_USERCODE	0x20
#define ERR_COMMACC		0x40
#define ERR_CONTEXT 	0x80
#define ERR_USERCONTEXT	0xA0	//combination of both, save code memory
#define ERR_USERSTRING	0x21	//combination of both, save code memory
#define ERR_CANCEL	    0x100
#define ERR_RUNTOKEN	0x200
#define ERR_NUMVAL		0x400
#define ERR_DIVZERO		0x800

#define KERNELSIZE		141
#define MAXUSER			168//*6 words p.struct=1008 words=4032 bytes=ca.2pages
#define MAXSTACK		64
#define MAXCODE			0xC00   // =3072 words =12288 bytes
                                //takes 6 pages each 2048 bytes
#define CODETOP			MAXCODE - 1
#define USERTOP			(uint32_t)&gfUserHeader[MAXUSER]

#define CDC_CLASS	0       //initalizes device as virtual COM port
#define MIDI_CLASS	1       //initalizes device as native MIDI interface

#define DEFAULT_NUMBASE	10 //Decimal, enter 16 for HEX
#define USART_RXBUFLEN	1024
#define DEFAULT_BAUD	11
#define USART_TX_INTERRUPT //no TX Int. is useful in some debugging situations
                            //interrupt driven TX buffer may overflow(much data)
                             //or debug information gets lost if Forth crashes

#define CTRL_B  2
#define CTRL_D  4
#define BKSP  8
#define TAB   9
#define LFF  10		//for Macintosh OS: simply exchange values of CRR and LFF
#define CRR  13		//then New_Line will be interpreted correctly
#define CTRL_R  18
#define ESC  27
#define SPACE  32
#define BACKSLASH 92

//bitmask shortcuts:
#define m0      1
#define m1      (1<<1)
#define m2      (1<<2)
#define m3      (1<<3)
#define m4      (1<<4)
#define m5      (1<<5)
#define m6      (1<<6)
#define m7      (1<<7)
#define m8      (1<<8)
#define m9      (1<<9)
#define m10     (1<<10)
#define m11     (1<<11)
#define m12     (1<<12)
#define m13     (1<<13)
#define m14     (1<<14)
#define m15     (1<<15)
#define m16     (1<<16)
#define m17     (1<<17)
#define m18     (1<<18)
#define m19     (1<<19)
#define m20     (1<<20)
#define m21     (1<<21)
#define m22     (1<<22)
#define m23     (1<<23)
#define m24     (1<<24)
#define m25     (1<<25)
#define m26     (1<<26)
#define m27     (1<<27)
#define m28     (1<<28)
#define m29     (1<<29)
#define m30     (1<<30)
#define m31     (1<<31)

#endif

// ##########################################################################

// Version changes:
//REVNUM 1: 18 June 2020 copied from project CForth-G431.
//	    USB related code parts deleted
//          Communication restricted to USART2 (driven by ST-LINK module)
//REVNUM2: 22June20: 2 CONSTants,2VARiables precompiled as User Token 
//  25June20: bugfix "W":parameters exchanged
//  VAL? renamed to VAL and may be compiled, too
//REVNUM3: 02July2020: some debug messages removed
//	bugfix and improvements in save and load project in Flash
//	default number base changed to Decimal
//	minor improvements en detail
//REVNUM4:  08July2020: TFREQ,PWM,TIME,PULS,RANDOM, I2C, StackFlush new
//			08July20: doCONST renamed doVALUE, doVAR renamed doADDR
//			11July20: P" and TX" bytes stored more compact
//          PWM1(PA6) and PWM2(PA7) independent setup but same TFREQ
//			16July20: indices of kernel operators restructured
//released 20July2020
//	REVNUM5: 
//	02Aug20	USB and USART transmit buffers overflow at 144MHz clock fixed
//	05Aug20	reorg. of op "Q" (message of system parameters)
//REVNUM6:  13Aug20	bugfix in interpretUserToken(): 
//			TIB pointer was not restored after background process
//			SPI init parameters added to output of "Q"
//			send0X32() new, code more compact
//	15Aug20	bugfix in interpretUserToken(): 
//			TIB pointer was not restored after background process
//			improvement of FREQ initialisation, printout with op "Q"
//			problem gfSpiClock initialisation fixed, SpiClock limit=2047
//			SPI and PWM storage in Flash reorganized
//	16Aug20	terminal text interpreter converts TAB into SPACE
//release 16Aug20
//REVNUM7:  01November2020
//	09Nov20	return value of "interpretUserToken()" inverted
//			return of "interpretKernelToken()" changed to "void"
//			return of "executeTCode()" changed to "void"
//			op NODBG introduced,background process not executed,higher speed
//			if User Button(PC13) is pulled low during startup, AUTOEXE->"none"
//			after then, the User Button is freely available for the project
//			subQuery() rejects (TIB bytes == 0), this simplifies scanNext()
//			return of scanNext() changed to "void", improvement of op_AUTO
// 	23Nov20	Kernel Ops DEBUG, CASE, <CASE new. ENDIF replaced by THEN
//release 03Dec20
//REVNUM 8:
//	05Dec20	executeTcode etc. optimized for runtime improvement
//			Z-BREAK renamed ZBREAK and doZ-BRK renamed doZBRK
//	06Dec20 IF()...LOOP() and doIF()...doLOOP() improved
//			for better runtime behaviour and better readability of MEM()
//			;(SEMIS) optimized
//	10Dec20	pullup/down switched off when I/O is output (save current)
//	12Dec20	user terminal communication during NO-DBG improved (CTRL_D)
//	15Dec20	parameter format of sendX8() changed to uint32_t
//	16Dec20	time operators reorg. (MS, DS are blocking, BLOCK removed)		
//          improvement of CTRL_B (unblock RX, MS and DS) behaviour
//release 28Dec20
//REVNUM 9:
// 20Jan21:	bugfix "RANA": assignment of ADC channels and number 
//release 22Jan21
//REVNUM 10:
//	MAY2021
//	development system changed to "Segger Embedded Studio for ARM 5.44"
//	no changes of source code and project setup necessary
//	op_REV behaviour modified: can be compiled now
//	start behaviour of op AUTO improved
//	19MAY21	Kernel ops ROLL,INJECT,U> added, 2SWAP,2DUP removed
//	22MAY21	%X and %D "Immediate,Background" now
//			op VAL reduced to Background Process value message
//	25May21	op_QUEST and AUTOEXE check moved to "popFlashProject" 
//			for better start of AUTOEXE when another project is loaded
//	30May21	PICK, ROLL, INJECT rearranged to Forth79 style
//			FS ("flush data stack") modified to "NO attributes"
//release 01Jun2021
//REVNUM 11: 12Aug2021
//12Aug21	bugfix DAC initialisation
//release 24Oct21
//REVNUM 12: 26 Oct 2021
//14Nov21	new:Stepper motor drive as background task(VARiables SPEED,STEPS)
//			bugfix load project with modified globals
//			oP_VAL improved (not restricted to STEP mode anymore)
//27Nov21	MOT1,MOT4,MOT2,N1,N2 new (background motor control)
//			PWM34 (PA1, PA15) new
//release 06Dec2022
//REVNUM13 16Dec2021
//			AUTOEXE bahaviour changed (-1 removed, index not increased)
//			gfUserToken[0] not deletable
//			code MOT1,MOT4,MOT2 improved
//			bugfix "popFlashProject", gfPWM1,2
//release 21Dec2021
//REVNUM14 08May2022
//08May22   %x renamed $, %d renamed %
//			preloaded V2 removed, now P1,P2,P3 preloaded
//16May22	op_PRINTSTACK modified (CRR removed)
//27May22	string output of op_USER modified
//28May22	gfTICKCOUNT -> gfMSCOUNT and gfTIXCOUNT (decouple TIX from  MS,DS)
//08June22	VAR and CONST replaced by VARCON,
//			op_REV compiles op_doREV instead of op_doADDR, saves runtime
//			op_VAL removed, op_TRACE new
//			P1,P2,P3 renamed to K1,K2,K3,K4
//			due to these changes, some reorganisation of code
//09June22	struct "gfUserToken" renamed to "gfUserHeader"
//11June22	op_MY()1,op_MY()2,op_MY()3,op_MY()4 added
//			bugfix interpretNumber(), parsing big negative numbers
//			kernel operator ARRAY added
//13June22	improvement of compileUserToken() and restore()
//							(gfTCode offset calculation made more transparent)
//15June22	background process behaviour of KEY? and KEY improved
//16June2	REPLACE and RESTORE modified, straightened
//release 17June2022
//REVNUM15:	18June2022
//22March23	kernel op "NEG" renamend to "+/-"
//			kernel op "=" renamend to "=="
//30Sept23	kernel op "%" renamend to "DZ"
//			kernel op "$" renamend to "HX"
//02Oct2	kernel ops UNTIL and correseponding doUNTIL added
//			not MIDI version: PICK replaced by ROLL
//			option "Cold Reset" (CTRL_R) removed
//			independent Watchdog removed
//07Oct23	"RANA"(raed analog in) improved, new: disableADC(),saves current
//			P renamed . , PH renamed .H , P" renamed .", .D  implemented new
//			kernel Op 'Q' renamed to '?'
//			>CASE and do>CASE implemented new
//07Oct23	numberBase saved in gerneral preset, always reactived at startup
//15Oct23	pin arrangement for PWM and MOT improved
//			kernel op '&' renamed to 'AND', kernel op '|' renamed to 'OR
//16Oct23	kernel ops NOP, TIME, PULS, TRACE removed
//			MOT code rearranged.Now MOT1,2,3 each 2 wire.Option 4wire removed
//			FREQ12 on timer2, FREQ34 on timer3 now
//17Oct23	USER output reformatted
//20Oct23	REPLACE and RESTORE improved
//21OCT23	REGW, REGBSET, REGBCLR, REGBTST stack parameters exchanged
//25OCT23	RANA renamed RADC, RDIG renamed RDIN ,WAIT renamed TIME
//26Oct23	kernel ops "<<" and ">>" replaced by single "SHIFT"
//			"VID" , "PID" combined to "VIDPID"
//28Oct23	CONTI, BREAK, ZBREAK optimized, doCONTI,doBREAK,doZBREAK removed
//			K1,K2,K3,K4 renamed V1,V2,V3,V4
//30Oct23	servSystem() reorganized:
//			countdown is checked inside servSystem() instead of before called
//			bugfix: now watchdog retriggered in showPreStep()
//			gSYSCOUNT refresh rate 20Hz now, gLEDCOUNT adjusted to 300ms
//09Nov23	order of Kernel Ops rearranged
//12Nov23	RP-1 and RP-4 only "COMPILING" flag now.To be used ahead of RETURN
//release 12Nov23
//REVNUM16:13Nov2023
//19DEC23	MEM attributes changed to A_BAK
//20Dec23	bugfix compileTCode: protection against CODETOP overflow
//21Dec23	digital port I/O improved
//released 25Dec23	
//REVNUM->17
//31Dec23	modification compilation user header with kernel name (op_colon)
//			new gfBackProcess=2: return to higher thread level with CTRL_R
//01Jan24	changed: internally gfAuto = target UserFunction + 1
//			op_MEM now sends block of 128bytes(32 words)
//			MOT1,2,3 related VARCONs MS1,2,3 renamed MSTP1,2,3
//			improvement countdown NSTP1,2,3 if stepper is not rotating
//04Jan24	PWM1-4 and MOT1-3 pin assignment reorganized
//			PWM4 removed, FREQ34-->FREQ3
//			op_MOTx pins changed/unified for all HW versions
//11Jan24	send0X32() modified: always with leading 0x even at small numbers
//			query() made void
//released 14Jan24
//REVNUM-->18
//18Jan24	bugfix op_SEE()
//19Jan24	decimal numbers can be entered with leading '%' at any muber base
//			see interpretNumber()
//08Feb24	kernelOps ITX1 and IRXN implemented. IRXAN renamed to IADRXN
//26Feb24	program flow IRXADN improved
//07March24	kernel op RADC: only 1 single sample used for readout
//09March24	reorg kernel op SAVE and LOAD more Forth typically
//12March24	I2C code made more failure-resistant
//released 14March24
//REVNUM-->19
//31March24	bugfix cforth.h: DZ and HX have IMMEDIATE flag now
//02Apr24	doVALUE renamed doVAL

// ******************************